package java.lang;

import java.util.HashMap;
import java.io.IOException;
import java.util.jar.JarInputStream;
import java.io.FileInputStream;
import java.security.AccessController;
import java.net.MalformedURLException;
import sun.net.www.ParseUtil;
import java.io.File;
import java.security.PrivilegedAction;
import java.util.jar.Attributes;
import java.lang.annotation.Annotation;
import sun.reflect.CallerSensitive;
import sun.reflect.Reflection;
import java.util.jar.Manifest;
import java.net.URL;
import java.util.Map;
import java.lang.reflect.AnnotatedElement;

public class Package implements AnnotatedElement
{
    private static Map<String, Package> pkgs;
    private static Map<String, URL> urls;
    private static Map<String, Manifest> mans;
    private final String pkgName;
    private final String specTitle;
    private final String specVersion;
    private final String specVendor;
    private final String implTitle;
    private final String implVersion;
    private final String implVendor;
    private final URL sealBase;
    private final transient ClassLoader loader;
    private transient Class<?> packageInfo;
    
    public String getName() {
        return this.pkgName;
    }
    
    public String getSpecificationTitle() {
        return this.specTitle;
    }
    
    public String getSpecificationVersion() {
        return this.specVersion;
    }
    
    public String getSpecificationVendor() {
        return this.specVendor;
    }
    
    public String getImplementationTitle() {
        return this.implTitle;
    }
    
    public String getImplementationVersion() {
        return this.implVersion;
    }
    
    public String getImplementationVendor() {
        return this.implVendor;
    }
    
    public boolean isSealed() {
        return this.sealBase != null;
    }
    
    public boolean isSealed(URL url) {
        return url.equals(this.sealBase);
    }
    
    public boolean isCompatibleWith(String s) throws NumberFormatException {
        if (this.specVersion == null || this.specVersion.length() < 1) {
            throw new NumberFormatException("Empty version string");
        }
        String[] split = this.specVersion.split("\\.", -1);
        int[] array = new int[split.length];
        int i = 0;
        while (i < split.length) {
            array[i] = Integer.parseInt(split[i]);
            if (array[i] < 0) {
                throw NumberFormatException.forInputString("" + array[i]);
            }
            ++i;
        }
        String[] split2 = s.split("\\.", -1);
        int[] array2 = new int[split2.length];
        int j = 0;
        while (j < split2.length) {
            array2[j] = Integer.parseInt(split2[j]);
            if (array2[j] < 0) {
                throw NumberFormatException.forInputString("" + array2[j]);
            }
            ++j;
        }
        int max = Math.max(array2.length, array.length);
        int k = 0;
        while (k < max) {
            int n = (k < array2.length) ? array2[k] : 0;
            int n2 = (k < array.length) ? array[k] : 0;
            if (n2 < n) {
                return false;
            }
            if (n2 > n) {
                return true;
            }
            ++k;
        }
        return true;
    }
    
    @CallerSensitive
    public static Package getPackage(String s) {
        ClassLoader classLoader = ClassLoader.getClassLoader(Reflection.getCallerClass());
        if (classLoader != null) {
            return classLoader.getPackage(s);
        }
        return getSystemPackage(s);
    }
    
    @CallerSensitive
    public static Package[] getPackages() {
        ClassLoader classLoader = ClassLoader.getClassLoader(Reflection.getCallerClass());
        if (classLoader != null) {
            return classLoader.getPackages();
        }
        return getSystemPackages();
    }
    
    static Package getPackage(Class<?> clazz) {
        String name = clazz.getName();
        int lastIndex = name.lastIndexOf(46);
        if (lastIndex != -1) {
            String substring = name.substring(0, lastIndex);
            ClassLoader classLoader = clazz.getClassLoader();
            if (classLoader != null) {
                return classLoader.getPackage(substring);
            }
            return getSystemPackage(substring);
        }
        else {
            return null;
        }
    }
    
    @Override
    public int hashCode() {
        return this.pkgName.hashCode();
    }
    
    @Override
    public String toString() {
        String specTitle = this.specTitle;
        String specVersion = this.specVersion;
        String string;
        if (specTitle != null && specTitle.length() > 0) {
            string = ", " + specTitle;
        }
        else {
            string = "";
        }
        String string2;
        if (specVersion != null && specVersion.length() > 0) {
            string2 = ", version " + specVersion;
        }
        else {
            string2 = "";
        }
        return "package " + this.pkgName + string + string2;
    }
    
    private Class<?> getPackageInfo() {
        if (this.packageInfo == null) {
            try {
                this.packageInfo = Class.forName(this.pkgName + ".package-info", false, this.loader);
            }
            catch (ClassNotFoundException ex) {
                class PackageInfoProxy
                {
                }
                this.packageInfo = Package.1PackageInfoProxy.class;
            }
        }
        return this.packageInfo;
    }
    
    @Override
    public <A extends Annotation> A getAnnotation(Class<A> clazz) {
        return this.getPackageInfo().getAnnotation(clazz);
    }
    
    @Override
    public boolean isAnnotationPresent(Class<? extends Annotation> annotationClass) {
        return super.isAnnotationPresent(annotationClass);
    }
    
    @Override
    public <A extends Annotation> A[] getAnnotationsByType(Class<A> clazz) {
        return this.getPackageInfo().getAnnotationsByType(clazz);
    }
    
    @Override
    public Annotation[] getAnnotations() {
        return this.getPackageInfo().getAnnotations();
    }
    
    @Override
    public <A extends Annotation> A getDeclaredAnnotation(Class<A> clazz) {
        return this.getPackageInfo().getDeclaredAnnotation(clazz);
    }
    
    @Override
    public <A extends Annotation> A[] getDeclaredAnnotationsByType(Class<A> clazz) {
        return this.getPackageInfo().getDeclaredAnnotationsByType(clazz);
    }
    
    @Override
    public Annotation[] getDeclaredAnnotations() {
        return this.getPackageInfo().getDeclaredAnnotations();
    }
    
    Package(String pkgName, String specTitle, String specVersion, String specVendor, String implTitle, String implVersion, String implVendor, URL sealBase, ClassLoader loader) {
        this.pkgName = pkgName;
        this.implTitle = implTitle;
        this.implVersion = implVersion;
        this.implVendor = implVendor;
        this.specTitle = specTitle;
        this.specVersion = specVersion;
        this.specVendor = specVendor;
        this.sealBase = sealBase;
        this.loader = loader;
    }
    
    private Package(String pkgName, Manifest manifest, URL url, ClassLoader loader) {
        String concat = pkgName.replace('.', '/').concat("/");
        String s = null;
        String specTitle = null;
        String specVersion = null;
        String specVendor = null;
        String implTitle = null;
        String implVersion = null;
        String implVendor = null;
        URL sealBase = null;
        Attributes attributes = manifest.getAttributes(concat);
        if (attributes != null) {
            specTitle = attributes.getValue(Attributes.Name.SPECIFICATION_TITLE);
            specVersion = attributes.getValue(Attributes.Name.SPECIFICATION_VERSION);
            specVendor = attributes.getValue(Attributes.Name.SPECIFICATION_VENDOR);
            implTitle = attributes.getValue(Attributes.Name.IMPLEMENTATION_TITLE);
            implVersion = attributes.getValue(Attributes.Name.IMPLEMENTATION_VERSION);
            implVendor = attributes.getValue(Attributes.Name.IMPLEMENTATION_VENDOR);
            s = attributes.getValue(Attributes.Name.SEALED);
        }
        Attributes mainAttributes = manifest.getMainAttributes();
        if (mainAttributes != null) {
            if (specTitle == null) {
                specTitle = mainAttributes.getValue(Attributes.Name.SPECIFICATION_TITLE);
            }
            if (specVersion == null) {
                specVersion = mainAttributes.getValue(Attributes.Name.SPECIFICATION_VERSION);
            }
            if (specVendor == null) {
                specVendor = mainAttributes.getValue(Attributes.Name.SPECIFICATION_VENDOR);
            }
            if (implTitle == null) {
                implTitle = mainAttributes.getValue(Attributes.Name.IMPLEMENTATION_TITLE);
            }
            if (implVersion == null) {
                implVersion = mainAttributes.getValue(Attributes.Name.IMPLEMENTATION_VERSION);
            }
            if (implVendor == null) {
                implVendor = mainAttributes.getValue(Attributes.Name.IMPLEMENTATION_VENDOR);
            }
            if (s == null) {
                s = mainAttributes.getValue(Attributes.Name.SEALED);
            }
        }
        if ("true".equalsIgnoreCase(s)) {
            sealBase = url;
        }
        this.pkgName = pkgName;
        this.specTitle = specTitle;
        this.specVersion = specVersion;
        this.specVendor = specVendor;
        this.implTitle = implTitle;
        this.implVersion = implVersion;
        this.implVendor = implVendor;
        this.sealBase = sealBase;
        this.loader = loader;
    }
    
    static Package getSystemPackage(String concat) {
        synchronized (Package.pkgs) {
            Package defineSystemPackage = (Package)Package.pkgs.get(concat);
            if (defineSystemPackage == null) {
                concat = concat.replace('.', '/').concat("/");
                String systemPackage0 = getSystemPackage0(concat);
                if (systemPackage0 != null) {
                    defineSystemPackage = defineSystemPackage(concat, systemPackage0);
                }
            }
            return defineSystemPackage;
        }
    }
    
    static Package[] getSystemPackages() {
        String[] systemPackages0 = getSystemPackages0();
        synchronized (Package.pkgs) {
            for (int i = 0; i < systemPackages0.length; ++i) {
                defineSystemPackage(systemPackages0[i], getSystemPackage0(systemPackages0[i]));
            }
            return (Package[])Package.pkgs.values().toArray(new Package[Package.pkgs.size()]);
        }
    }
    
    private static Package defineSystemPackage(final String s, final String s2) {
        return (Package)AccessController.doPrivileged(new PrivilegedAction<Package>() {
            @Override
            public Package run() {
                String val$iname = s;
                URL fileToEncodedURL = (URL)Package.urls.get(s2);
                if (fileToEncodedURL == null) {
                    File file = new File(s2);
                    try {
                        fileToEncodedURL = ParseUtil.fileToEncodedURL(file);
                    }
                    catch (MalformedURLException ex) {}
                    if (fileToEncodedURL != null) {
                        Package.urls.put(s2, fileToEncodedURL);
                        if (file.isFile()) {
                            Package.mans.put(s2, Package.access$100(s2));
                        }
                    }
                }
                String replace = iname.substring(0, iname.length() - 1).replace('/', '.');
                Manifest manifest = (Manifest)Package.mans.get(s2);
                Object o;
                if (manifest != null) {
                    o = new Package(replace, manifest, fileToEncodedURL);
                }
                else {
                    o = new Package(replace, null, null, null, null, null, null, null, null);
                }
                Package.pkgs.put(replace, o);
                return o;
            }
        });
    }
    
    private static Manifest loadManifest(String name) {
        try (final FileInputStream in = new FileInputStream(name);
             final JarInputStream jarInputStream = new JarInputStream(in, false)) {
            return jarInputStream.getManifest();
        }
        catch (IOException ex) {
            return null;
        }
    }
    
    private static native String getSystemPackage0(String p0);
    
    private static native String[] getSystemPackages0();
    
    static Manifest access$100(String s) {
        return loadManifest(s);
    }
    
    Package(String s, Manifest manifest, URL url, ClassLoader classLoader, Package.1 privilegedAction) {
        this(s, manifest, url, classLoader);
    }
    
    static {
        Package.pkgs = new HashMap(31);
        Package.urls = new HashMap(10);
        Package.mans = new HashMap(10);
    }
}
